import 'dart:io';

import 'package:camera/camera.dart';

import 'package:flutter/material.dart';
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:skeleton_animation/skeleton_animation.dart';

class FBImagePicker extends StatefulWidget {
  const FBImagePicker({
    final Key? key,
    this.title,
    this.enableCamera = true,
    this.btnColor = Colors.grey,
    this.backgroundColor = Colors.white,
    this.textStyle = const TextStyle(
      color: Colors.black,
      fontSize: 16,
    ),
    required this.onComplete,
    required this.afterNextBTN,
    required this.backOnSystem,
  }) : super(key: key);

  final String? title;

  /// enable Camera ( default is true )
  final bool enableCamera;

  /// set btn color ( default Colors.grey )

  final Color btnColor;

  /// set background color ( default Colors.white )
  final Color backgroundColor;

  /// set text style
  final TextStyle textStyle;

  /// Callback that returns list path picker.
  final ValueChanged<List> onComplete;

  /// function after next BTN
  final VoidCallback? afterNextBTN;

  /// set back Action in sysytem and back icon
  final VoidCallback backOnSystem;

  @override
  FBImagePickerState createState() => FBImagePickerState();
}

class FBImagePickerState extends State<FBImagePicker> {
  bool skeltone = true; // show skelton before loaded
  List allFilePath = [];
  List listfinal = [];
  List typeFormat = ['.jpg', '.png'];
  List<String> folderName = []; // list of folder name(have image)
  List<List> folderFilePath = []; // list of image separate by folder name
  List folderfile = []; // just use filemaneger function
  int indexfolderShow = 0; // index name show folder
  bool runCamera = false; // show camera
  late CameraController cameraController; // camera controller
  int cameraindex = 0; // list of camera
  bool takePhotoReady = false; // show taked image
  XFile? imageFile; // taked image file
  List<int> _selectedItems = [];
  bool shownext = false; // show next button when selected
  bool showCategory = false;
  bool colorSelected = false;
  bool haveMedia = false;
  bool cameraInitial = false;
  bool flashOuto = false;
  BoxDecoration selected = BoxDecoration(
    border: Border.all(color: Colors.blue, width: 3),
  );
  BoxDecoration unselected = const BoxDecoration();
  List cameras = [];

  @override
  void initState() {
    Future.delayed(const Duration(seconds: 2), filemanager);
    super.initState();
  }

  Future<void> getCamera() async {
    if (cameras.isEmpty) {
      if (await Permission.camera.request().isGranted) {
        cameras = await availableCameras();
      }
    }
    initialCamera();
  }

  void initialCamera() {
    if (cameras.isNotEmpty) {
      cameraController = CameraController(
        cameras[cameraindex], // set camera
        ResolutionPreset.max, // set resolotion
        enableAudio: false, // disable audio
        imageFormatGroup: ImageFormatGroup.jpeg, // image format
      );
      cameraController.initialize().then((final _) {
        if (!mounted) {
          return;
        }
        debugPrint('initial Camera');
        runCamera = true;
        cameraInitial = true;
        setState(() {});
      });
    } else {
      getCamera();
    }
  }

  Future<void> takePhoto() async {
    imageFile = await cameraController.takePicture(); // take photo from camera
    setState(() {
      takePhotoReady = true; // show taked image
    });
  }

  List filetype() {
    return typeFormat = ['.jpg', '.png'];
  }

  Future<List?> filemanager() async {
    if (await Permission.storage.request().isGranted) {
      filetype();
      setState(() {
        skeltone = true; // show skelton
      });
      allFilePath = []; // clear object
      folderName = []; // clear object
      folderFilePath = []; // clear object
      folderfile = []; // clear object
      final folderHaveFile = <FileSystemEntity>[]; // list for folder in storage
      var storages =
          (await getExternalStorageDirectories())!; //  get storage addres
      storages = storages.map((final Directory e) {
        final splitedPath = e.path.split('/');
        return Directory(
          splitedPath
              .sublist(
                0,
                splitedPath.indexWhere((final element) => element == 'Android'),
              )
              .join('/'),
        );
      }).toList(); // *** get directory list
      for (var l = 0; l < storages.length; l++) {
        // get image list from both storage
        final direct = '${storages[l].path}/';
        final myDir = Directory(direct);
        var folders = myDir.listSync(followLinks: false);
        for (var i = 1; i < folders.length; i++) {
          // filter folder list
          if (folders[i].path.split('/').last == 'DCIM' ||
              folders[i].path.split('/').last == 'Pictures' ||
              folders[i].path.split('/').last == 'Picture' ||
              folders[i].path.split('/').last == 'Media' ||
              folders[i].path.split('/').last == 'Gallery' ||
              folders[i].path.split('/').last == 'gallery' ||
              folders[i].path.split('/').last == 'Music' ||
              folders[i].path.split('/').last == 'MUSIC' ||
              folders[i].path.split('/').last == 'Podcast' ||
              folders[i].path.split('/').last == 'Doc' ||
              folders[i].path.split('/').last == 'DOC' ||
              folders[i].path.split('/').last == 'doc' ||
              folders[i].path.split('/').last == 'DOCUMENTS' ||
              folders[i].path.split('/').last == 'Documents' ||
              folders[i].path.split('/').last == 'DOCUMENTS' ||
              folders[i].path.split('/').last == 'Movies' ||
              folders[i].path.split('/').last == 'Video' ||
              folders[i].path.split('/').last == 'video' ||
              folders[i].path.split('/').last == 'VIDEO' ||
              folders[i].path.split('/').last == 'Download' ||
              folders[i].path.split('/').last == 'Image' ||
              folders[i].path.split('/').last == 'Photo') {
            folderHaveFile.add(folders[i]);
          }
        }
        folders = folderHaveFile;
        for (var folderIndex = 0; folderIndex < folders.length; folderIndex++) {
          if (folders[folderIndex] is Directory) {
            // get directory in the list
            final directoryList = Directory('${folders[folderIndex].path}/')
                .listSync(recursive: true, followLinks: false);
            for (var k = 0; k < directoryList.length; k++) {
              //// remove .thumbnails folder
              for (var w = 0;
                  w < directoryList[k].path.split('/').length;
                  w++) {
                if (directoryList[k].path.split('/')[w] == '.thumbnails') {
                  directoryList.remove(directoryList[k]);
                  k -= 1;
                  if (k < 0) {
                    k = 0;
                  }
                }
              }
            }
            if (directoryList.isNotEmpty) {
              for (var typeIndex = 0;
                  typeIndex < typeFormat.length;
                  typeIndex++) {
                // get file with type format (like image = .jpf .png)
                for (var listIndex = 0;
                    listIndex < directoryList.length;
                    listIndex++) {
                  if (extension(directoryList[listIndex].path) ==
                      typeFormat[typeIndex]) {
                    allFilePath.add(
                      directoryList[listIndex].path,
                    );
                    folderName.add(
                      // get folder name
                      File(directoryList[listIndex].path)
                          .parent
                          .path
                          .split('/')
                          .last,
                    );
                  }
                }
              }
            }
          }
        }
      }
      if (folderName.isNotEmpty) {
        haveMedia = true;
        folderName = folderName.toSet().toList(); // remove duplicate name
        for (var k = 0; k < folderName.length; k++) {
          folderfile = [];
          for (var j = 0; j < allFilePath.length; j++) {
            if (File(allFilePath[j]).parent.path.split('/').last ==
                folderName[k]) {
              folderfile.add(allFilePath[j]); // create list of image path}

            }
          }
          folderfile = folderfile.toSet().toList(); // remove duplicate image
          folderFilePath.add(folderfile);
        }
        folderName.insert(0, 'Gallery ');
        folderFilePath.insert(0, allFilePath);
      } else {
        setState(() {
          haveMedia = false;
          skeltone = false;
        });
      }
      setState(() {
        skeltone = false; // show skelton
      });
      return folderfile;
    } else {
      await Permission.storage.request(); // get storage permission
      return null;
    }
  }

  @override
  void dispose() {
    if (cameraInitial) {
      debugPrint('dispose Camera');
      cameraController.dispose();
    } // dispose camera
    super.dispose();
  }

  void onCompleteNext() {
    listfinal = [];
    for (var j = 0; j < _selectedItems.toList().length; j++) {
      listfinal
          .add(folderFilePath[indexfolderShow][_selectedItems.toList()[j]]);
    }
    widget.onComplete(
      listfinal,
    );
    widget.afterNextBTN!();
  }

  void onCompleteCameraNext() {
    listfinal = [];
    final file = File(imageFile!.path);
    listfinal.add(file.path);
    widget.onComplete(
      listfinal,
    );
    widget.afterNextBTN!();
  }

  @override
  Widget build(final BuildContext context) {
    return WillPopScope(
        onWillPop: () {
          widget.backOnSystem();
          return Future.value(false);
        },
        child: SafeArea(
          child: runCamera // show camera
              ? Scaffold(
                  backgroundColor: Colors.black,
                  appBar: takePhotoReady
                      ? AppBar(
                          backgroundColor: Colors.white,
                          // bottomOpacity: 0,
                          elevation: 2,
                          leading: IconButton(
                            // back
                            icon: const Icon(
                              Icons.arrow_back_sharp,
                              color: Colors.black,
                            ),
                            onPressed: () {
                              setState(() {
                                takePhotoReady = false;
                              });
                            },
                          ),
                          title: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              Text(
                                'Camera',
                                style: widget.textStyle,
                              ),
                              TextButton(
                                onPressed: onCompleteCameraNext,
                                child: Container(
                                  padding: const EdgeInsets.only(
                                    left: 8,
                                    right: 8,
                                    top: 10,
                                    bottom: 10,
                                  ),
                                  child: Text(
                                    'Next',
                                    style: widget.textStyle,
                                  ),
                                ),
                              )
                            ],
                          ),
                        )
                      : null,
                  body: takePhotoReady // show taked photo
                      ? SizedBox(
                          height: MediaQuery.of(context).size.height,
                          width: MediaQuery.of(context).size.width,
                          child: Image.file(
                            File(imageFile!.path),
                            fit: BoxFit.cover,
                          ),
                        )
                      : SizedBox(
                          height: MediaQuery.of(context).size.height,
                          width: MediaQuery.of(context).size.width,
                          child: CameraPreview(
                            cameraController, // show camera
                            child: Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                Padding(
                                  padding: const EdgeInsets.all(10),
                                  child: Row(
                                    mainAxisAlignment:
                                        MainAxisAlignment.spaceBetween,
                                    children: [
                                      InkWell(
                                        // back to image picker
                                        onTap: () {
                                          setState(() {
                                            runCamera = false;
                                          });
                                        },
                                        child: Container(
                                          alignment: Alignment.center,
                                          padding: const EdgeInsets.all(10),
                                          decoration: BoxDecoration(
                                            color: Colors.grey.withOpacity(0.2),
                                            shape: BoxShape.circle,
                                          ),
                                          child: const Icon(
                                            Icons.close,
                                            size: 30,
                                            color: Colors.white,
                                          ),
                                        ),
                                      ),
                                      InkWell(
                                        // run flash
                                        onTap: () {
                                          if (!flashOuto) {
                                            cameraController
                                                .setFlashMode(FlashMode.always);
                                            flashOuto = true;
                                          } else {
                                            cameraController
                                                .setFlashMode(FlashMode.off);
                                            flashOuto = false;
                                          }
                                          setState(() {});
                                        },
                                        child: Container(
                                          alignment: Alignment.center,
                                          padding: const EdgeInsets.all(10),
                                          decoration: BoxDecoration(
                                            color: Colors.grey.withOpacity(0.2),
                                            shape: BoxShape.circle,
                                          ),
                                          child: Icon(
                                            Icons.flash_on,
                                            size: 30,
                                            color: flashOuto
                                                ? Colors.yellow
                                                : Colors.white,
                                          ),
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                                const Spacer(),
                                Padding(
                                  padding: const EdgeInsets.all(10),
                                  child: Row(
                                    mainAxisAlignment:
                                        MainAxisAlignment.spaceBetween,
                                    children: [
                                      InkWell(
                                        // back to image picker
                                        onTap: () {
                                          setState(() {
                                            runCamera = false;
                                          });
                                        },
                                        child: Container(
                                          alignment: Alignment.center,
                                          padding: const EdgeInsets.all(10),
                                          decoration: BoxDecoration(
                                            color: Colors.grey.withOpacity(0.2),
                                            shape: BoxShape.circle,
                                          ),
                                          child: const Icon(
                                            Icons.browse_gallery_outlined,
                                            size: 30,
                                            color: Colors.white,
                                          ),
                                        ),
                                      ),
                                      InkWell(
                                        // take photo
                                        onTap: takePhoto,
                                        child: Container(
                                          alignment: Alignment.center,
                                          padding: const EdgeInsets.all(10),
                                          decoration: BoxDecoration(
                                            color: Colors.grey.withOpacity(0.2),
                                            shape: BoxShape.circle,
                                          ),
                                          child: const Icon(
                                            Icons.camera,
                                            size: 50,
                                            color: Colors.white,
                                          ),
                                        ),
                                      ),
                                      InkWell(
                                        // change camera
                                        onTap: () {
                                          if (cameraindex == 0) {
                                            cameraindex = 1;
                                          } else {
                                            cameraindex = 0;
                                          }
                                          setState(() {});
                                          initialCamera();
                                        },
                                        child: Container(
                                          alignment: Alignment.center,
                                          padding: const EdgeInsets.all(10),
                                          decoration: BoxDecoration(
                                            color: Colors.grey.withOpacity(0.2),
                                            shape: BoxShape.circle,
                                          ),
                                          child: const Icon(
                                            Icons.cameraswitch_outlined,
                                            size: 30,
                                            color: Colors.white,
                                          ),
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                              ],
                            ),
                          ),
                        ))
              : Scaffold(
                  backgroundColor: widget.backgroundColor,
                  appBar: AppBar(
                    backgroundColor: widget.backgroundColor,
                    // bottomOpacity: 0,
                    elevation: 2,
                    leading: IconButton(
                      // back
                      icon: Icon(
                        Icons.arrow_back_sharp,
                        color: Colors.blue[900],
                      ),
                      onPressed: widget.backOnSystem,
                    ),
                    title: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        Text(
                          widget.title ?? '',
                          style: widget.textStyle,
                        ),
                        if (shownext && !showCategory)
                          TextButton(
                            onPressed: onCompleteNext,
                            child: Container(
                              padding: const EdgeInsets.only(
                                left: 8,
                                right: 8,
                                top: 10,
                                bottom: 10,
                              ),
                              child: Text(
                                'Next',
                                style: widget.textStyle,
                              ),
                            ),
                          )
                      ],
                    ),
                  ),
                  body: Container(
                    alignment: Alignment.center,
                    child: skeltone // show skeltone before get file list
                        ? GridView.count(
                            crossAxisCount: 3,
                            children: List.generate(100, (final index) {
                              return Padding(
                                  padding: const EdgeInsets.all(3),
                                  child: Skeleton(
                                    animation: SkeletonAnimation.pulse,
                                  ));
                            }),
                          )
                        : Column(
                            // show image list
                            children: [
                              Padding(
                                padding: const EdgeInsets.only(
                                    top: 24, left: 16, right: 18),
                                child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: [
                                    InkWell(
                                      onTap: () {
                                        if (haveMedia) {
                                          if (showCategory) {
                                            showCategory = false;
                                          } else {
                                            showCategory = true;
                                          }
                                          setState(() {});
                                        }
                                      },
                                      child: Container(
                                        width: 119,
                                        decoration: BoxDecoration(
                                          color: widget.btnColor,
                                          borderRadius:
                                              BorderRadius.circular(8),
                                        ),
                                        child: Padding(
                                          padding: const EdgeInsets.all(10),
                                          child: Row(
                                            children: [
                                              Expanded(
                                                child: Text(
                                                  haveMedia
                                                      ? folderName[
                                                          indexfolderShow]
                                                      : 'No Media',
                                                  maxLines: 1,
                                                  overflow:
                                                      TextOverflow.ellipsis,
                                                ),
                                              ),
                                              Icon(
                                                showCategory
                                                    ? Icons
                                                        .keyboard_arrow_up_rounded
                                                    : Icons
                                                        .keyboard_arrow_down_rounded,
                                                size: 20,
                                              )
                                            ],
                                          ),
                                        ),
                                      ),
                                    ),
                                    const Spacer(),
                                    if (!showCategory)
                                      InkWell(
                                        // show selection
                                        onTap: () {
                                          if (_selectedItems.isNotEmpty) {
                                            _selectedItems = [];
                                            colorSelected = false;
                                            shownext = false;
                                          }
                                          setState(() {});
                                        },
                                        child: Container(
                                          alignment: Alignment.center,
                                          height: 40,
                                          margin: const EdgeInsets.all(16),
                                          padding: const EdgeInsets.all(10),
                                          decoration: BoxDecoration(
                                            borderRadius:
                                                const BorderRadius.all(
                                              Radius.circular(8),
                                            ),
                                            color: colorSelected
                                                ? Colors.blue[900]
                                                : widget.btnColor,
                                          ),
                                          child: const Icon(
                                            Icons.markunread_mailbox_outlined,
                                            color: Colors.white,
                                            size: 20,
                                          ),
                                        ),
                                      ),
                                    if (!showCategory && widget.enableCamera)
                                      InkWell(
                                        // show camera
                                        onTap: initialCamera,
                                        child: Container(
                                          alignment: Alignment.center,
                                          height: 40,
                                          padding: const EdgeInsets.all(10),
                                          decoration: BoxDecoration(
                                            borderRadius:
                                                const BorderRadius.all(
                                              Radius.circular(8),
                                            ),
                                            color: widget.btnColor,
                                          ),
                                          child: const Icon(
                                            Icons.photo_camera_sharp,
                                            color: Colors.white,
                                            size: 20,
                                          ),
                                        ),
                                      ),
                                  ],
                                ),
                              ),
                              if (showCategory)
                                Expanded(
                                  child: APPFolderImageListView(
                                    textStyle: widget.textStyle,
                                    backgroundColor: widget.backgroundColor,
                                    folderName: folderName,
                                    folderIntoPath: folderFilePath,
                                    onComplete: (final value) {
                                      for (var jk = 0;
                                          jk < folderName.length;
                                          jk++) {
                                        if (value == folderName[jk]) {
                                          indexfolderShow = jk;
                                          _selectedItems = [];
                                          shownext = false;
                                          colorSelected = false;
                                        }
                                      }
                                      showCategory = false;
                                      setState(() {});
                                    },
                                  ),
                                )
                              else
                                Expanded(
                                  child: haveMedia
                                      ? GridView.count(
                                          // show image list
                                          crossAxisCount: 3,
                                          children: List.generate(
                                              folderFilePath[indexfolderShow]
                                                  .length, (final index) {
                                            return Container(
                                              decoration:
                                                  _selectedItems.contains(
                                                index,
                                              ) // selected file decoration
                                                      ? selected
                                                      : unselected,
                                              margin: const EdgeInsets.all(1),
                                              alignment: Alignment.center,
                                              child: InkWell(
                                                // get back image selected path
                                                onTap: () {
                                                  _selectedItems.contains(index)
                                                      ? _selectedItems
                                                          .removeWhere(
                                                          (final val) =>
                                                              val == index,
                                                        ) // selected file
                                                      : _selectedItems.add(
                                                          index,
                                                        ); // unselected file
                                                  if (_selectedItems.isEmpty) {
                                                    shownext = false;
                                                    colorSelected = false;
                                                  } else {
                                                    shownext =
                                                        true; // show next btn
                                                    colorSelected = true;
                                                  }
                                                  setState(() {});
                                                },
                                                child: Stack(
                                                  fit: StackFit.expand,
                                                  children: [
                                                    Image.file(
                                                      File(
                                                        folderFilePath[
                                                                indexfolderShow]
                                                            [index],
                                                      ),
                                                      fit: BoxFit.fill,
                                                      cacheHeight: 200,
                                                      cacheWidth: 200,
                                                      errorBuilder: (
                                                        // image show error
                                                        final BuildContext
                                                            context,
                                                        final Object exception,
                                                        final StackTrace?
                                                            stackTrace,
                                                      ) {
                                                        return const Center(
                                                          child: Text(
                                                            'Load error',
                                                          ),
                                                        );
                                                      },
                                                    ),
                                                    if (_selectedItems
                                                        .contains(index))
                                                      Positioned(
                                                        top: 0,
                                                        right: 4,
                                                        child: Container(
                                                          padding:
                                                              const EdgeInsets
                                                                  .all(
                                                            6,
                                                          ),
                                                          alignment:
                                                              Alignment.center,
                                                          decoration:
                                                              const BoxDecoration(
                                                            color: Colors.blue,
                                                            shape:
                                                                BoxShape.circle,
                                                          ),
                                                          child: Text(
                                                            (_selectedItems
                                                                        .indexWhere(
                                                                      (final element) =>
                                                                          element ==
                                                                          index,
                                                                    ) +
                                                                    1)
                                                                .toString(), //  show number of selected
                                                            style:
                                                                const TextStyle(
                                                              color:
                                                                  Colors.white,
                                                              fontSize: 14,
                                                            ),
                                                          ),
                                                        ),
                                                      )
                                                    else
                                                      const Text(
                                                        '',
                                                        style: TextStyle(
                                                          color: Colors.white,
                                                          fontSize: 0,
                                                        ),
                                                      ),
                                                  ],
                                                ),
                                              ),
                                            );
                                          }),
                                        )
                                      : Text(
                                          'No Media',
                                          style: widget.textStyle,
                                          maxLines: 1,
                                          overflow: TextOverflow.ellipsis,
                                        ),
                                ),
                            ],
                          ),
                  ),
                ),
        ));
  }
}

class APPFolderImageListView extends StatefulWidget {
  const APPFolderImageListView({
    final Key? key,
    required this.backgroundColor,
    required this.folderName,
    required this.folderIntoPath,
    required this.textStyle,
    this.onComplete,
  }) : super(key: key);

  /// folders name
  final List<String> folderName;

  /// folders into path
  final List<List<dynamic>> folderIntoPath;

  final Color? backgroundColor;

  final TextStyle? textStyle;

  /// Selected directory
  final ValueChanged<String>? onComplete;

  @override
  State<APPFolderImageListView> createState() => APPFolderImageListViewState();
}

class APPFolderImageListViewState extends State<APPFolderImageListView> {
  @override
  Widget build(final BuildContext context) {
    return Container(
      height: MediaQuery.of(context).size.height,
      width: MediaQuery.of(context).size.width,
      decoration: BoxDecoration(
        color: widget.backgroundColor,
      ),
      child: Column(
        children: [
          const Padding(padding: EdgeInsets.all(10), child: Divider()),
          Expanded(
            child: ListView.builder(
              itemCount: widget.folderName.length,
              itemBuilder: (final context, final index) {
                return Padding(
                  padding:
                      const EdgeInsets.only(left: 16, right: 16, bottom: 42),
                  child: InkWell(
                    onTap: () {
                      widget.onComplete!(widget.folderName[index]);
                    },
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            Text(
                              widget.folderName[index],
                              style: widget.textStyle,
                            ),
                            Text(
                              widget.folderIntoPath[index].length.toString(),
                              style: TextStyle(color: Colors.grey[400]),
                            ),
                          ],
                        ),
                        // if (widget.folderIntoPath[index][0] is Uint8List)
                        Container(
                          height: 64,
                          width: 64,
                          decoration: BoxDecoration(
                            borderRadius:
                                const BorderRadius.all(Radius.circular(8)),
                            image: DecorationImage(
                              image: FileImage(
                                File(
                                  widget.folderIntoPath[index][0],
                                ),
                              ),
                              fit: BoxFit.fill,
                              onError: (
                                final Object exception,
                                final StackTrace? stackTrace,
                              ) {
                                Center(
                                  child: Text(
                                    '😢 load image',
                                    style: widget.textStyle,
                                  ),
                                );
                              },
                              filterQuality: FilterQuality.medium,
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                );
              },
            ),
          )
        ],
      ),
    );
  }
}
